# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'trnsysGUI/hydraulicLoops/_dialogs/edit/_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.3
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_hydraulicLoopDialog(object):
    def setupUi(self, hydraulicLoopDialog):
        hydraulicLoopDialog.setObjectName("hydraulicLoopDialog")
        hydraulicLoopDialog.resize(554, 562)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(hydraulicLoopDialog.sizePolicy().hasHeightForWidth())
        hydraulicLoopDialog.setSizePolicy(sizePolicy)
        self.verticalLayout = QtWidgets.QVBoxLayout(hydraulicLoopDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.generalGroupBox = QtWidgets.QGroupBox(hydraulicLoopDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.generalGroupBox.sizePolicy().hasHeightForWidth())
        self.generalGroupBox.setSizePolicy(sizePolicy)
        self.generalGroupBox.setObjectName("generalGroupBox")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.generalGroupBox)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.loopNameLabel = QtWidgets.QLabel(self.generalGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.loopNameLabel.sizePolicy().hasHeightForWidth())
        self.loopNameLabel.setSizePolicy(sizePolicy)
        self.loopNameLabel.setObjectName("loopNameLabel")
        self.horizontalLayout_2.addWidget(self.loopNameLabel)
        self.loopName = QtWidgets.QLineEdit(self.generalGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.loopName.sizePolicy().hasHeightForWidth())
        self.loopName.setSizePolicy(sizePolicy)
        self.loopName.setObjectName("loopName")
        self.horizontalLayout_2.addWidget(self.loopName)
        self.invalidNameWarningWidget = QtWidgets.QWidget(self.generalGroupBox)
        self.invalidNameWarningWidget.setMinimumSize(QtCore.QSize(22, 22))
        self.invalidNameWarningWidget.setStyleSheet("image: url(:/tango-icons/status/tango-icon-theme-0.8.90/scalable/status/dialog-warning.svg);")
        self.invalidNameWarningWidget.setObjectName("invalidNameWarningWidget")
        self.horizontalLayout_2.addWidget(self.invalidNameWarningWidget)
        spacerItem = QtWidgets.QSpacerItem(0, 19, QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.fluidLabel = QtWidgets.QLabel(self.generalGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.fluidLabel.sizePolicy().hasHeightForWidth())
        self.fluidLabel.setSizePolicy(sizePolicy)
        self.fluidLabel.setObjectName("fluidLabel")
        self.horizontalLayout_2.addWidget(self.fluidLabel)
        self.fluidComboBox = QtWidgets.QComboBox(self.generalGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.fluidComboBox.sizePolicy().hasHeightForWidth())
        self.fluidComboBox.setSizePolicy(sizePolicy)
        self.fluidComboBox.setObjectName("fluidComboBox")
        self.horizontalLayout_2.addWidget(self.fluidComboBox)
        spacerItem1 = QtWidgets.QSpacerItem(130, 19, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.verticalLayout_4.addLayout(self.horizontalLayout_2)
        self.useLoopWideDefaultsCheckBox = QtWidgets.QCheckBox(self.generalGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.useLoopWideDefaultsCheckBox.sizePolicy().hasHeightForWidth())
        self.useLoopWideDefaultsCheckBox.setSizePolicy(sizePolicy)
        self.useLoopWideDefaultsCheckBox.setObjectName("useLoopWideDefaultsCheckBox")
        self.verticalLayout_4.addWidget(self.useLoopWideDefaultsCheckBox)
        self.verticalLayout.addWidget(self.generalGroupBox)
        self.bulkOperationsGroupBox = QtWidgets.QGroupBox(hydraulicLoopDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.bulkOperationsGroupBox.sizePolicy().hasHeightForWidth())
        self.bulkOperationsGroupBox.setSizePolicy(sizePolicy)
        self.bulkOperationsGroupBox.setMinimumSize(QtCore.QSize(0, 180))
        self.bulkOperationsGroupBox.setStyleSheet("")
        self.bulkOperationsGroupBox.setObjectName("bulkOperationsGroupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.bulkOperationsGroupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.widget = QtWidgets.QWidget(self.bulkOperationsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.widget.sizePolicy().hasHeightForWidth())
        self.widget.setSizePolicy(sizePolicy)
        self.widget.setMinimumSize(QtCore.QSize(20, 20))
        self.widget.setStyleSheet("image: url(:/tango-icons/status/tango-icon-theme-0.8.90/scalable/status/dialog-information.svg);")
        self.widget.setObjectName("widget")
        self.horizontalLayout_4.addWidget(self.widget)
        self.label_6 = QtWidgets.QLabel(self.bulkOperationsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_6.sizePolicy().hasHeightForWidth())
        self.label_6.setSizePolicy(sizePolicy)
        self.label_6.setMinimumSize(QtCore.QSize(0, 20))
        self.label_6.setLineWidth(0)
        self.label_6.setWordWrap(True)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_4.addWidget(self.label_6)
        self.verticalLayout_2.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.bulkLengthTypeComboBox = QtWidgets.QComboBox(self.bulkOperationsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.bulkLengthTypeComboBox.sizePolicy().hasHeightForWidth())
        self.bulkLengthTypeComboBox.setSizePolicy(sizePolicy)
        self.bulkLengthTypeComboBox.setObjectName("bulkLengthTypeComboBox")
        self.bulkLengthTypeComboBox.addItem("")
        self.bulkLengthTypeComboBox.addItem("")
        self.gridLayout.addWidget(self.bulkLengthTypeComboBox, 0, 0, 1, 1)
        self.bulkLengthLineEdit = QtWidgets.QLineEdit(self.bulkOperationsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.bulkLengthLineEdit.sizePolicy().hasHeightForWidth())
        self.bulkLengthLineEdit.setSizePolicy(sizePolicy)
        self.bulkLengthLineEdit.setObjectName("bulkLengthLineEdit")
        self.gridLayout.addWidget(self.bulkLengthLineEdit, 0, 1, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(self.bulkOperationsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.bulkLengthInfoIconWidget = QtWidgets.QWidget(self.bulkOperationsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.bulkLengthInfoIconWidget.sizePolicy().hasHeightForWidth())
        self.bulkLengthInfoIconWidget.setSizePolicy(sizePolicy)
        self.bulkLengthInfoIconWidget.setMinimumSize(QtCore.QSize(20, 20))
        self.bulkLengthInfoIconWidget.setBaseSize(QtCore.QSize(0, 0))
        self.bulkLengthInfoIconWidget.setStyleSheet("image: url(:/tango-icons/status/tango-icon-theme-0.8.90/scalable/status/dialog-information.svg);")
        self.bulkLengthInfoIconWidget.setObjectName("bulkLengthInfoIconWidget")
        self.horizontalLayout.addWidget(self.bulkLengthInfoIconWidget)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.gridLayout.addLayout(self.horizontalLayout, 0, 2, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.bulkOperationsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_4.sizePolicy().hasHeightForWidth())
        self.label_4.setSizePolicy(sizePolicy)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 1, 0, 1, 1)
        self.bulkDiameterLineEdit = QtWidgets.QLineEdit(self.bulkOperationsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.bulkDiameterLineEdit.sizePolicy().hasHeightForWidth())
        self.bulkDiameterLineEdit.setSizePolicy(sizePolicy)
        self.bulkDiameterLineEdit.setObjectName("bulkDiameterLineEdit")
        self.gridLayout.addWidget(self.bulkDiameterLineEdit, 1, 1, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.bulkOperationsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_5.sizePolicy().hasHeightForWidth())
        self.label_5.setSizePolicy(sizePolicy)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 1, 2, 1, 1)
        self.bulkLengthUnitLabel = QtWidgets.QLabel(self.bulkOperationsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.bulkLengthUnitLabel.sizePolicy().hasHeightForWidth())
        self.bulkLengthUnitLabel.setSizePolicy(sizePolicy)
        self.bulkLengthUnitLabel.setObjectName("bulkLengthUnitLabel")
        self.gridLayout.addWidget(self.bulkLengthUnitLabel, 2, 0, 1, 1)
        self.bulkUValueLineEdit = QtWidgets.QLineEdit(self.bulkOperationsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.bulkUValueLineEdit.sizePolicy().hasHeightForWidth())
        self.bulkUValueLineEdit.setSizePolicy(sizePolicy)
        self.bulkUValueLineEdit.setObjectName("bulkUValueLineEdit")
        self.gridLayout.addWidget(self.bulkUValueLineEdit, 2, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.bulkOperationsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
        self.label_3.setSizePolicy(sizePolicy)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 2, 2, 1, 1)
        self.horizontalLayout_5.addLayout(self.gridLayout)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem3)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.bulkApplyButton = QtWidgets.QPushButton(self.bulkOperationsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.bulkApplyButton.sizePolicy().hasHeightForWidth())
        self.bulkApplyButton.setSizePolicy(sizePolicy)
        self.bulkApplyButton.setMaximumSize(QtCore.QSize(75, 24))
        self.bulkApplyButton.setObjectName("bulkApplyButton")
        self.horizontalLayout_3.addWidget(self.bulkApplyButton)
        self.bulkCancelButton = QtWidgets.QPushButton(self.bulkOperationsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.bulkCancelButton.sizePolicy().hasHeightForWidth())
        self.bulkCancelButton.setSizePolicy(sizePolicy)
        self.bulkCancelButton.setMaximumSize(QtCore.QSize(75, 24))
        self.bulkCancelButton.setObjectName("bulkCancelButton")
        self.horizontalLayout_3.addWidget(self.bulkCancelButton)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem4)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.verticalLayout.addWidget(self.bulkOperationsGroupBox)
        self.pipesGroupBox = QtWidgets.QGroupBox(hydraulicLoopDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pipesGroupBox.sizePolicy().hasHeightForWidth())
        self.pipesGroupBox.setSizePolicy(sizePolicy)
        self.pipesGroupBox.setObjectName("pipesGroupBox")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.pipesGroupBox)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.connectionsTableView = QtWidgets.QTableView(self.pipesGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.connectionsTableView.sizePolicy().hasHeightForWidth())
        self.connectionsTableView.setSizePolicy(sizePolicy)
        self.connectionsTableView.setObjectName("connectionsTableView")
        self.connectionsTableView.horizontalHeader().setStretchLastSection(True)
        self.verticalLayout_3.addWidget(self.connectionsTableView)
        self.pipesStatisticsLabel = QtWidgets.QLabel(self.pipesGroupBox)
        self.pipesStatisticsLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.pipesStatisticsLabel.setObjectName("pipesStatisticsLabel")
        self.verticalLayout_3.addWidget(self.pipesStatisticsLabel)
        self.verticalLayout.addWidget(self.pipesGroupBox)
        self.okCancelButtonBox = QtWidgets.QDialogButtonBox(hydraulicLoopDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.okCancelButtonBox.sizePolicy().hasHeightForWidth())
        self.okCancelButtonBox.setSizePolicy(sizePolicy)
        self.okCancelButtonBox.setOrientation(QtCore.Qt.Horizontal)
        self.okCancelButtonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.okCancelButtonBox.setObjectName("okCancelButtonBox")
        self.verticalLayout.addWidget(self.okCancelButtonBox)

        self.retranslateUi(hydraulicLoopDialog)
        self.okCancelButtonBox.accepted.connect(hydraulicLoopDialog.accept)
        self.okCancelButtonBox.rejected.connect(hydraulicLoopDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(hydraulicLoopDialog)

    def retranslateUi(self, hydraulicLoopDialog):
        _translate = QtCore.QCoreApplication.translate
        hydraulicLoopDialog.setWindowTitle(_translate("hydraulicLoopDialog", "Hydraulic loop properties"))
        self.generalGroupBox.setTitle(_translate("hydraulicLoopDialog", "Loop properties"))
        self.loopNameLabel.setText(_translate("hydraulicLoopDialog", "Name"))
        self.invalidNameWarningWidget.setToolTip(_translate("hydraulicLoopDialog", "<html><head/><body><p>The name is already taken</p></body></html>"))
        self.fluidLabel.setText(_translate("hydraulicLoopDialog", "Fluid"))
        self.useLoopWideDefaultsCheckBox.setText(_translate("hydraulicLoopDialog", "Use loop-wide defaults for pipe properties"))
        self.bulkOperationsGroupBox.setTitle(_translate("hydraulicLoopDialog", "Bulk pipe values assignment"))
        self.label_6.setText(_translate("hydraulicLoopDialog", "Values will be applied to all pipes. Unspecified properties will be left unchanged."))
        self.bulkLengthTypeComboBox.setItemText(0, _translate("hydraulicLoopDialog", "Total loop length"))
        self.bulkLengthTypeComboBox.setItemText(1, _translate("hydraulicLoopDialog", "Individual pipe length"))
        self.label.setText(_translate("hydraulicLoopDialog", "m"))
        self.bulkLengthInfoIconWidget.setToolTip(_translate("hydraulicLoopDialog", "<html><head/><body><p>The total length will be distributed evenly among the pipes</p></body></html>"))
        self.label_4.setText(_translate("hydraulicLoopDialog", "Diameter"))
        self.label_5.setText(_translate("hydraulicLoopDialog", "cm"))
        self.bulkLengthUnitLabel.setText(_translate("hydraulicLoopDialog", "U-Value"))
        self.label_3.setText(_translate("hydraulicLoopDialog", "W/(m^2 K)"))
        self.bulkApplyButton.setText(_translate("hydraulicLoopDialog", "Apply"))
        self.bulkCancelButton.setText(_translate("hydraulicLoopDialog", "Clear"))
        self.pipesGroupBox.setTitle(_translate("hydraulicLoopDialog", "Pipes"))
        self.pipesStatisticsLabel.setText(_translate("hydraulicLoopDialog", "Number of pipes: 5, total loop length: x"))
