# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'trnsysGUI/hydraulicLoops/_dialogs/split/_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.3
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_splitHydraulicLoopDialog(object):
    def setupUi(self, splitHydraulicLoopDialog):
        splitHydraulicLoopDialog.setObjectName("splitHydraulicLoopDialog")
        splitHydraulicLoopDialog.resize(353, 209)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(splitHydraulicLoopDialog.sizePolicy().hasHeightForWidth())
        splitHydraulicLoopDialog.setSizePolicy(sizePolicy)
        splitHydraulicLoopDialog.setMaximumSize(QtCore.QSize(16777215, 209))
        self.verticalLayout = QtWidgets.QVBoxLayout(splitHydraulicLoopDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.widget = QtWidgets.QWidget(splitHydraulicLoopDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.widget.sizePolicy().hasHeightForWidth())
        self.widget.setSizePolicy(sizePolicy)
        self.widget.setMinimumSize(QtCore.QSize(20, 20))
        self.widget.setStyleSheet("image: url(:/tango-icons/status/tango-icon-theme-0.8.90/scalable/status/dialog-information.svg);")
        self.widget.setObjectName("widget")
        self.horizontalLayout_3.addWidget(self.widget)
        self.label = QtWidgets.QLabel(splitHydraulicLoopDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setWordWrap(True)
        self.label.setObjectName("label")
        self.horizontalLayout_3.addWidget(self.label)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.loop1GroupBox = EnterLeaveNotifyingGroupBox(splitHydraulicLoopDialog)
        self.loop1GroupBox.setObjectName("loop1GroupBox")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.loop1GroupBox)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.loopNameLabel = QtWidgets.QLabel(self.loop1GroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.loopNameLabel.sizePolicy().hasHeightForWidth())
        self.loopNameLabel.setSizePolicy(sizePolicy)
        self.loopNameLabel.setObjectName("loopNameLabel")
        self.horizontalLayout.addWidget(self.loopNameLabel)
        self.name1ComboBox = QtWidgets.QComboBox(self.loop1GroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.name1ComboBox.sizePolicy().hasHeightForWidth())
        self.name1ComboBox.setSizePolicy(sizePolicy)
        self.name1ComboBox.setMinimumSize(QtCore.QSize(100, 0))
        self.name1ComboBox.setEditable(True)
        self.name1ComboBox.setObjectName("name1ComboBox")
        self.horizontalLayout.addWidget(self.name1ComboBox)
        self.name1WarningWidget = QtWidgets.QWidget(self.loop1GroupBox)
        self.name1WarningWidget.setMinimumSize(QtCore.QSize(30, 30))
        self.name1WarningWidget.setStyleSheet("image: url(:/tango-icons/status/tango-icon-theme-0.8.90/scalable/status/dialog-warning.svg);")
        self.name1WarningWidget.setObjectName("name1WarningWidget")
        self.horizontalLayout.addWidget(self.name1WarningWidget)
        spacerItem = QtWidgets.QSpacerItem(30, 20, QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.fluidLabel = QtWidgets.QLabel(self.loop1GroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.fluidLabel.sizePolicy().hasHeightForWidth())
        self.fluidLabel.setSizePolicy(sizePolicy)
        self.fluidLabel.setObjectName("fluidLabel")
        self.horizontalLayout.addWidget(self.fluidLabel)
        self.fluid1ComboBox = QtWidgets.QComboBox(self.loop1GroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.fluid1ComboBox.sizePolicy().hasHeightForWidth())
        self.fluid1ComboBox.setSizePolicy(sizePolicy)
        self.fluid1ComboBox.setMinimumSize(QtCore.QSize(100, 0))
        self.fluid1ComboBox.setObjectName("fluid1ComboBox")
        self.horizontalLayout.addWidget(self.fluid1ComboBox)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout.addWidget(self.loop1GroupBox)
        self.loop2GroupBox = EnterLeaveNotifyingGroupBox(splitHydraulicLoopDialog)
        self.loop2GroupBox.setObjectName("loop2GroupBox")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.loop2GroupBox)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.loopNameLabel_2 = QtWidgets.QLabel(self.loop2GroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.loopNameLabel_2.sizePolicy().hasHeightForWidth())
        self.loopNameLabel_2.setSizePolicy(sizePolicy)
        self.loopNameLabel_2.setObjectName("loopNameLabel_2")
        self.horizontalLayout_2.addWidget(self.loopNameLabel_2)
        self.name2ComboBox = QtWidgets.QComboBox(self.loop2GroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.name2ComboBox.sizePolicy().hasHeightForWidth())
        self.name2ComboBox.setSizePolicy(sizePolicy)
        self.name2ComboBox.setMinimumSize(QtCore.QSize(100, 0))
        self.name2ComboBox.setEditable(True)
        self.name2ComboBox.setObjectName("name2ComboBox")
        self.horizontalLayout_2.addWidget(self.name2ComboBox)
        self.name2WarningWidget = QtWidgets.QWidget(self.loop2GroupBox)
        self.name2WarningWidget.setMinimumSize(QtCore.QSize(30, 30))
        self.name2WarningWidget.setStyleSheet("image: url(:/tango-icons/status/tango-icon-theme-0.8.90/scalable/status/dialog-warning.svg);")
        self.name2WarningWidget.setObjectName("name2WarningWidget")
        self.horizontalLayout_2.addWidget(self.name2WarningWidget)
        spacerItem2 = QtWidgets.QSpacerItem(30, 20, QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem2)
        self.fluidLabel_2 = QtWidgets.QLabel(self.loop2GroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.fluidLabel_2.sizePolicy().hasHeightForWidth())
        self.fluidLabel_2.setSizePolicy(sizePolicy)
        self.fluidLabel_2.setObjectName("fluidLabel_2")
        self.horizontalLayout_2.addWidget(self.fluidLabel_2)
        self.fluid2ComboBox = QtWidgets.QComboBox(self.loop2GroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.fluid2ComboBox.sizePolicy().hasHeightForWidth())
        self.fluid2ComboBox.setSizePolicy(sizePolicy)
        self.fluid2ComboBox.setMinimumSize(QtCore.QSize(100, 0))
        self.fluid2ComboBox.setObjectName("fluid2ComboBox")
        self.horizontalLayout_2.addWidget(self.fluid2ComboBox)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.verticalLayout.addWidget(self.loop2GroupBox)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem4)
        self.applyButton = QtWidgets.QPushButton(splitHydraulicLoopDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.applyButton.sizePolicy().hasHeightForWidth())
        self.applyButton.setSizePolicy(sizePolicy)
        self.applyButton.setObjectName("applyButton")
        self.horizontalLayout_5.addWidget(self.applyButton)
        self.abortButton = QtWidgets.QPushButton(splitHydraulicLoopDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.abortButton.sizePolicy().hasHeightForWidth())
        self.abortButton.setSizePolicy(sizePolicy)
        self.abortButton.setObjectName("abortButton")
        self.horizontalLayout_5.addWidget(self.abortButton)
        self.verticalLayout.addLayout(self.horizontalLayout_5)

        self.retranslateUi(splitHydraulicLoopDialog)
        QtCore.QMetaObject.connectSlotsByName(splitHydraulicLoopDialog)

    def retranslateUi(self, splitHydraulicLoopDialog):
        _translate = QtCore.QCoreApplication.translate
        splitHydraulicLoopDialog.setWindowTitle(_translate("splitHydraulicLoopDialog", "Split hydraulic loop"))
        self.label.setText(_translate("splitHydraulicLoopDialog", "Hover over the designated area of a given loop to highlight its connections."))
        self.loop1GroupBox.setTitle(_translate("splitHydraulicLoopDialog", "Loop 1"))
        self.loopNameLabel.setText(_translate("splitHydraulicLoopDialog", "Name"))
        self.fluidLabel.setText(_translate("splitHydraulicLoopDialog", "Fluid"))
        self.loop2GroupBox.setTitle(_translate("splitHydraulicLoopDialog", "Loop 2"))
        self.loopNameLabel_2.setText(_translate("splitHydraulicLoopDialog", "Name"))
        self.fluidLabel_2.setText(_translate("splitHydraulicLoopDialog", "Fluid"))
        self.applyButton.setText(_translate("splitHydraulicLoopDialog", "Apply"))
        self.abortButton.setText(_translate("splitHydraulicLoopDialog", "Abort"))
from ._widgets import EnterLeaveNotifyingGroupBox
