# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'trnsysGUI/hydraulicLoops/_dialogs/merge/_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.3
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_mergeLoopsDialog(object):
    def setupUi(self, mergeLoopsDialog):
        mergeLoopsDialog.setObjectName("mergeLoopsDialog")
        mergeLoopsDialog.resize(340, 128)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(mergeLoopsDialog.sizePolicy().hasHeightForWidth())
        mergeLoopsDialog.setSizePolicy(sizePolicy)
        mergeLoopsDialog.setMinimumSize(QtCore.QSize(0, 0))
        mergeLoopsDialog.setMaximumSize(QtCore.QSize(16777215, 128))
        self.verticalLayout = QtWidgets.QVBoxLayout(mergeLoopsDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.differingFluidsWarningWidget = QtWidgets.QWidget(mergeLoopsDialog)
        self.differingFluidsWarningWidget.setMinimumSize(QtCore.QSize(30, 30))
        self.differingFluidsWarningWidget.setStyleSheet("image: url(:/tango-icons/status/tango-icon-theme-0.8.90/scalable/status/dialog-warning.svg);")
        self.differingFluidsWarningWidget.setObjectName("differingFluidsWarningWidget")
        self.horizontalLayout_2.addWidget(self.differingFluidsWarningWidget)
        self.differingFluidsWarningLabel = QtWidgets.QLabel(mergeLoopsDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.differingFluidsWarningLabel.sizePolicy().hasHeightForWidth())
        self.differingFluidsWarningLabel.setSizePolicy(sizePolicy)
        self.differingFluidsWarningLabel.setWordWrap(True)
        self.differingFluidsWarningLabel.setObjectName("differingFluidsWarningLabel")
        self.horizontalLayout_2.addWidget(self.differingFluidsWarningLabel)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.loopNameLabel = QtWidgets.QLabel(mergeLoopsDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.loopNameLabel.sizePolicy().hasHeightForWidth())
        self.loopNameLabel.setSizePolicy(sizePolicy)
        self.loopNameLabel.setObjectName("loopNameLabel")
        self.horizontalLayout.addWidget(self.loopNameLabel)
        self.nameComboBox = QtWidgets.QComboBox(mergeLoopsDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.nameComboBox.sizePolicy().hasHeightForWidth())
        self.nameComboBox.setSizePolicy(sizePolicy)
        self.nameComboBox.setMinimumSize(QtCore.QSize(100, 0))
        self.nameComboBox.setEditable(True)
        self.nameComboBox.setObjectName("nameComboBox")
        self.horizontalLayout.addWidget(self.nameComboBox)
        self.invalidNameWarningWidget = QtWidgets.QWidget(mergeLoopsDialog)
        self.invalidNameWarningWidget.setMinimumSize(QtCore.QSize(30, 30))
        self.invalidNameWarningWidget.setStyleSheet("image: url(:/tango-icons/status/tango-icon-theme-0.8.90/scalable/status/dialog-warning.svg);")
        self.invalidNameWarningWidget.setObjectName("invalidNameWarningWidget")
        self.horizontalLayout.addWidget(self.invalidNameWarningWidget)
        spacerItem = QtWidgets.QSpacerItem(30, 20, QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.fluidLabel = QtWidgets.QLabel(mergeLoopsDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.fluidLabel.sizePolicy().hasHeightForWidth())
        self.fluidLabel.setSizePolicy(sizePolicy)
        self.fluidLabel.setObjectName("fluidLabel")
        self.horizontalLayout.addWidget(self.fluidLabel)
        self.fluidComboBox = QtWidgets.QComboBox(mergeLoopsDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.fluidComboBox.sizePolicy().hasHeightForWidth())
        self.fluidComboBox.setSizePolicy(sizePolicy)
        self.fluidComboBox.setMinimumSize(QtCore.QSize(100, 0))
        self.fluidComboBox.setObjectName("fluidComboBox")
        self.horizontalLayout.addWidget(self.fluidComboBox)
        self.noFluidSelectedWarningWidget = QtWidgets.QWidget(mergeLoopsDialog)
        self.noFluidSelectedWarningWidget.setMinimumSize(QtCore.QSize(30, 30))
        self.noFluidSelectedWarningWidget.setStyleSheet("image: url(:/tango-icons/status/tango-icon-theme-0.8.90/scalable/status/dialog-warning.svg);")
        self.noFluidSelectedWarningWidget.setObjectName("noFluidSelectedWarningWidget")
        self.horizontalLayout.addWidget(self.noFluidSelectedWarningWidget)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem2)
        self.applyButton = QtWidgets.QPushButton(mergeLoopsDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.applyButton.sizePolicy().hasHeightForWidth())
        self.applyButton.setSizePolicy(sizePolicy)
        self.applyButton.setObjectName("applyButton")
        self.horizontalLayout_5.addWidget(self.applyButton)
        self.abortButton = QtWidgets.QPushButton(mergeLoopsDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.abortButton.sizePolicy().hasHeightForWidth())
        self.abortButton.setSizePolicy(sizePolicy)
        self.abortButton.setObjectName("abortButton")
        self.horizontalLayout_5.addWidget(self.abortButton)
        self.verticalLayout.addLayout(self.horizontalLayout_5)

        self.retranslateUi(mergeLoopsDialog)
        QtCore.QMetaObject.connectSlotsByName(mergeLoopsDialog)

    def retranslateUi(self, mergeLoopsDialog):
        _translate = QtCore.QCoreApplication.translate
        mergeLoopsDialog.setWindowTitle(_translate("mergeLoopsDialog", "Hydraulic loops merge"))
        self.differingFluidsWarningLabel.setText(_translate("mergeLoopsDialog", "The fluids of the loops you are about to merge are different. Please choose the fluid for the merged loop or abort."))
        self.loopNameLabel.setText(_translate("mergeLoopsDialog", "Name"))
        self.fluidLabel.setText(_translate("mergeLoopsDialog", "Fluid"))
        self.noFluidSelectedWarningWidget.setToolTip(_translate("mergeLoopsDialog", "<html><head/><body><p>A fluid must be selected</p></body></html>"))
        self.applyButton.setText(_translate("mergeLoopsDialog", "Apply"))
        self.abortButton.setText(_translate("mergeLoopsDialog", "Abort"))
